@@ -74,6 +74,7 @@ GetOverlappingGenes <- function(sc.eset, bulk.eset, markers, verbose) {
 #'   that indicates cell type
 #' @return sc.ref Matrix. Reference profile with number of gene rows by number
 #'   of cell types columns. 
+#' @export
 GenerateSCReference <- function(sc.eset, cell.types) {
   cell.labels <- base::factor(sc.eset[[cell.types]])
   all.cell.types <- base::levels(cell.labels)
@@ -97,6 +98,7 @@ GenerateSCReference <- function(sc.eset, cell.types) {
 #'   that indicates cell type
 #' @return sc.props Matrix. Cell proportions with number of cell types rows 
 #'   by number of individuals columns
+#' @export
 CalculateSCCellProportions <- function(sc.eset, subject.names, cell.types) {
   individual.labels <- base::factor(sc.eset[[subject.names]])
   individuals <- base::levels(individual.labels)
@@ -234,14 +236,26 @@ SemisupervisedTransformBulk <- function(gene, Y.train, X.pred) {
 #' @param use.overlap Boolean. Whether to use and expect overlapping samples 
 #'   in decomposition.
 #' @param verbose Boolean. Whether to print log info during decomposition.
-#'   Errors will be printed regardless. 
+#'   Errors will be printed regardless.
+#' @param old.cpm Prior to version 1.0.4 (updated in July 2020), the package
+#'   converted counts to CPM after subsetting the marker genes. Github user
+#'   randel pointed out that the order of these operations should be switched.
+#'   Thanks randel! This option is provided for replication of older BisqueRNA
+#'   but should be enabled, especially for small marker gene sets.
+#'   We briefly tested this change on the cortex and adipose datasets.
+#'   The original and new order of operations produce estimates that have an
+#'   average correlation of 0.87 for the cortex and 0.84 for the adipose within
+#'   each cell type.
 #' @return A list. Slot \strong{bulk.props} contains a matrix of cell type
 #'   proportion estimates with cell types as rows and individuals as columns.
 #'   Slot \strong{sc.props} contains a matrix of cell type proportions 
 #'   estimated directly from counting single-cell data. 
 #'   Slot \strong{rnorm} contains Euclidean norm of the residuals for each
 #'   individual's proportion estimates. Slot \strong{genes.used} contains
-#'   vector of genes used in decomposition
+#'   vector of genes used in decomposition. Slot \strong{transformed.bulk}
+#'   contains the transformed bulk expression used for decomposition. These
+#'   values are generated by applying a linear transformation to the CPM
+#'   expression.
 #' @examples
 #' library(Biobase)
 #' sim.data <- SimulateData(n.ind=10, n.genes=100, n.cells=100,
@@ -258,7 +272,8 @@ ReferenceBasedDecomposition <- function(bulk.eset,
                                         cell.types="cellType",
                                         subject.names="SubjectName",
                                         use.overlap=TRUE, 
-                                        verbose=TRUE) {
+                                        verbose=TRUE,
+                                        old.cpm=TRUE) {
   if ((! methods::is(sc.eset, "ExpressionSet")) || 
       (! methods::is(bulk.eset, "ExpressionSet"))) {
     base::stop("Expression data should be in ExpressionSet")
@@ -302,23 +317,35 @@ ReferenceBasedDecomposition <- function(bulk.eset,
     markers <- base::unique(base::unlist(markers))
   }
   genes <- GetOverlappingGenes(sc.eset, bulk.eset, markers, verbose)
-  sc.eset <-
-    Biobase::ExpressionSet(assayData=Biobase::exprs(sc.eset)[genes,],
-                           phenoData=sc.eset@phenoData)
-  bulk.eset <-
-    Biobase::ExpressionSet(assayData=Biobase::exprs(bulk.eset)[genes,],
-                           phenoData=bulk.eset@phenoData)
+  if (old.cpm) {
+    sc.eset <-
+      Biobase::ExpressionSet(assayData=Biobase::exprs(sc.eset)[genes,],
+                             phenoData=sc.eset@phenoData)
+    bulk.eset <-
+      Biobase::ExpressionSet(assayData=Biobase::exprs(bulk.eset)[genes,],
+                             phenoData=bulk.eset@phenoData)
+  }
   if (verbose) {
     base::message("Converting single-cell counts to CPM and ",
               "filtering zero variance genes.")
   }
   sc.eset <- CountsToCPM(sc.eset)
+  if (!old.cpm) {
+    sc.eset <-
+      Biobase::ExpressionSet(assayData=Biobase::exprs(sc.eset)[genes,],
+                             phenoData=sc.eset@phenoData)
+  }
   sc.eset <- FilterZeroVarianceGenes(sc.eset, verbose)
   if (verbose) {
     base::message("Converting bulk counts to CPM and filtering",
                   " unexpressed genes.")
   }
   bulk.eset <- CountsToCPM(bulk.eset)
+  if (!old.cpm) {
+    bulk.eset <-
+      Biobase::ExpressionSet(assayData=Biobase::exprs(bulk.eset)[genes,],
+                             phenoData=bulk.eset@phenoData)
+  }
   bulk.eset <- FilterUnexpressedGenes(bulk.eset, verbose)
   genes <- base::intersect(Biobase::featureNames(sc.eset),
                            Biobase::featureNames(bulk.eset))
@@ -413,10 +440,14 @@ ReferenceBasedDecomposition <- function(bulk.eset,
   base::rownames(results) <- base::append(base::colnames(sc.ref), "rnorm")
   base::colnames(results) <- sample.names
   rnorm <- results["rnorm",,drop=T]
-  names(rnorm) <- sample.names
+  base::names(rnorm) <- sample.names
+  Y.pred <- base::t(Y.pred) 
+  base::rownames(Y.pred) <- base::rownames(sc.ref)
+  base::colnames(Y.pred) <- sample.names
   results <- base::list(bulk.props=results[base::colnames(sc.ref),,drop=F],
                         sc.props=sc.props,
                         rnorm=rnorm,
-                        genes.used=base::rownames(sc.ref))
+                        genes.used=base::rownames(sc.ref),
+                        transformed.bulk=Y.pred)
   return(results)
 }
